@@ -16,6 +16,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
+#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -95,6 +96,19 @@ static int make_tmp_directory(pam_handle_t *pamh) {
   int ret = 0;
   int status;
   pid_t pid;
+  struct sigaction old, new;
+
+  new.sa_handler = SIG_DFL;
+  sigemptyset(&new.sa_mask);
+  new.sa_flags = 0;
+
+  /* save old signal handler */
+  ret = sigaction(SIGCHLD, &new, &old);
+  if (ret == -1) {
+    _log_err(LOG_WARNING, "could not save SIGCHLD handler");
+    return 1;
+  }
+
   /* Do fork and exec by hand.. */
   pid = fork();
   if (!pid) {
@@ -102,8 +116,21 @@ static int make_tmp_directory(pam_handle_t *pamh) {
     if (geteuid() == 0)
       setuid(get_user_id(pamh));
     execl(PAM_TMPDIR_HELPER,PAM_TMPDIR_HELPER,NULL);
-    }
-  wait(&status);
+  }
+
+  ret = waitpid(pid, &status, 0);
+  if (ret == -1) {
+    _log_err(LOG_ERR, "error waiting for helper binary (%d)\n", errno);
+    return 1;
+  }
+
+  /* restore old signal handler */
+  ret = sigaction(SIGCHLD, &old, NULL);
+  if (ret == -1) {
+    _log_err(LOG_WARNING, "could not restore SIGCHLD handler");
+    return 1;
+  }
+
   return WEXITSTATUS(status);
 }
 
